#include "ipv4.h"

#define ANSI_COLOR_RED     "\x1b[31m"
#define ANSI_COLOR_GREEN   "\x1b[32m"
#define ANSI_COLOR_YELLOW  "\x1b[33m"
#define ANSI_COLOR_BLUE    "\x1b[34m"
#define ANSI_COLOR_MAGENTA "\x1b[35m"
#define ANSI_COLOR_CYAN    "\x1b[36m"
#define ANSI_COLOR_RESET   "\x1b[0m"

const char * usage = "Usage: ./ipv4 [{{ip1} {sm1} {ip2} {sm2} | {generate subnets}]\n";

void testNetworkNumber
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
    uint32_t ip1_i, ip2_i, sm1_i, sm2_i;
    char * network1, * network2;

    ip1_i = parse(ip1);
    ip2_i = parse(ip2);
    sm1_i = parse(sm1);
    sm2_i = parse(sm2);

    isValidNetmask(sm1_i);
    isValidNetmask(sm2_i);

    network1 = calloc(sizeof(char), 16);
    network2 = calloc(sizeof(char), 16);
    ipntoa(networkNumber(ip1_i, sm1_i), network1, 16);
    ipntoa(networkNumber(ip2_i, sm2_i), network2, 16);

    printf("                NETWORK NUMBERS\n");
    printf("IP                Subnet Mask       Prefix  Network Number   Class\n");
    printf("%-15s   %-15s   %-2d      %-15s  %c\n", ip1, sm1, prefixLength(sm1_i), network1, classful(ip1_i));
    printf("%-15s   %-15s   %-2d      %-15s  %c\n", ip2, sm2, prefixLength(sm2_i), network2, classful(ip2_i));

    free(network1);
    free(network2);
}

void testSubnet
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  uint32_t ip1_i, ip2_i, sm1_i, sm2_i, bc1, bc2;
  char * broadcast1, * broadcast2;

    broadcast1 = calloc(sizeof(char), 16);
    broadcast2 = calloc(sizeof(char), 16);

    ip1_i = parse(ip1);
    ip2_i = parse(ip2);
    sm1_i = parse(sm1);
    sm2_i = parse(sm2);

    bc1 = subnetDirectedBroadcast(ip1_i, sm1_i);
    bc2 = subnetDirectedBroadcast(ip2_i, sm2_i);
    ipntoa(bc1, broadcast1, 16);
    ipntoa(bc2, broadcast2, 16);

    printf("             SUBNET DIRECTED BROADCAST\n");
    printf("IP                Subnet Mask       Prefix  Broadcast\n");
    printf("%-15s   %-15s   %-2d      %-15s\n", ip1, sm1, prefixLength(sm1_i), broadcast1);
    printf("%-15s   %-15s   %-2d      %-15s\n", ip2, sm2, prefixLength(sm2_i), broadcast2);

    free(broadcast1);
    free(broadcast2);
}

void testNetworkSize
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  uint32_t ip1_i, ip2_i, sm1_i, sm2_i, net1_i, net2_i;
  char * cidr1, * cidr2, * net1_s, * net2_s;

  cidr1 = calloc(sizeof(char), 20);
  cidr2 = calloc(sizeof(char), 20);

  ip1_i = parse(ip1);
  ip2_i = parse(ip2);
  sm1_i = parse(sm1);
  sm2_i = parse(sm2);

  net1_i = networkNumber(ip1_i, sm1_i);
  net2_i = networkNumber(ip2_i, sm2_i);
  net1_s = ipntoa(net1_i, NULL, 16);
  net2_s = ipntoa(net2_i, NULL, 16);

  snprintf(cidr1, 20, "%s/%d", net1_s, prefixLength(sm1_i));
  snprintf(cidr2, 20, "%s/%d", net2_s, prefixLength(sm2_i));

  printf("                  NETWORK SIZE\n");
  printf("Network\t\t\tSubnets\t\tHosts\n");
  printf("%-18s\t%d\t\t%d\n", cidr1, countSubnets(ip1_i, sm1_i), countHosts(sm1_i));
  printf("%-18s\t%d\t\t%d\n", cidr2, countSubnets(ip2_i, sm2_i), countHosts(sm2_i));

  free(net1_s);
  free(net2_s);
  free(cidr1);
  free(cidr2);
}

void testSummary
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  uint32_t ip1_i, ip2_i, sm1_i, sm2_i, sumpfx;
    char * sip, * ssm, * wcm;

    ip1_i = parse(ip1);
    ip2_i = parse(ip2);
    sm1_i = parse(sm1);
    sm2_i = parse(sm2);

    sip = calloc(sizeof(char), 16);
    ssm = calloc(sizeof(char), 16);
    wcm = calloc(sizeof(char), 16);

    sumpfx = summaryPrefix(ip1_i, sm1_i, ip2_i, sm2_i);

    ipntoa(summaryAddress(ip1_i, sm1_i, ip2_i, sm2_i), sip, 16);
    ipntoa(sumpfx, ssm, 16);
    ipntoa(wildcardMask(sumpfx), wcm, 16);

    printf("                 SUMMARY ROUTE\n");
    printf("Summary IP        Summary Mask      Prefix   Wildcard Mask\n");
    printf("%-15s   %-15s   %-2d       %-15s\n", sip, ssm, prefixLength(sumpfx), wcm);
    free(sip);
    free(ssm);
    free(wcm);
}

void testAll
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
    testNetworkNumber(ip1, sm1, ip2, sm2);
    printf("\n");
    testSubnet(ip1, sm1, ip2, sm2);
    printf("\n");
    testNetworkSize(ip1, sm1, ip2, sm2);
    printf("\n");
    testSummary(ip1, sm1, ip2, sm2);
}

int main
(int argc, char ** argv)
{
    if (argc == 5)
    {
      testAll(argv[1], argv[2], argv[3], argv[4]);
    }
    else if (argc== 3 && (strncmp(argv[1], "generate", 10) == 0) && 
	     (strncmp(argv[2], "subnets", 8) == 0))
      {
	generateSubnets();
      }
    else
    {
      fprintf(stderr, ANSI_COLOR_YELLOW "%s\n" ANSI_COLOR_RESET, usage);
      testAll("198.51.100.4", "255.255.255.224", "198.51.100.77", "255.255.255.248");
    }

    return(0);
}
